@@ -46,15 +46,18 @@
 #include "shared/cairo-util.h"
 #include "shared/config-parser.h"
 #include "shared/helpers.h"
+#include "shared/xalloc.h"
+#include "shared/zalloc.h"
 
-#include "desktop-shell-client-protocol.h"
+#include "weston-desktop-shell-client-protocol.h"
+
+#define DEFAULT_CLOCK_FORMAT CLOCK_FORMAT_MINUTES
 
 extern char **environ; /* defined by libc */
 
 struct desktop {
 	struct display *display;
-	struct desktop_shell *shell;
-	uint32_t interface_version;
+	struct weston_desktop_shell *shell;
 	struct unlock_dialog *unlock_dialog;
 	struct task unlock_task;
 	struct wl_list outputs;
@@ -72,7 +75,7 @@ struct desktop {
 
 struct surface {
 	void (*configure)(void *data,
-			  struct desktop_shell *desktop_shell,
+			  struct weston_desktop_shell *desktop_shell,
 			  uint32_t edges, struct window *window,
 			  int32_t width, int32_t height);
 };
@@ -84,6 +87,7 @@ struct panel {
 	struct wl_list launcher_list;
 	struct panel_clock *clock;
 	int painted;
+	int clock_format;
 	uint32_t color;
 };
 
@@ -123,6 +127,8 @@ struct panel_clock {
 	struct panel *panel;
 	struct task clock_task;
 	int clock_fd;
+	char *format_string;
+	time_t refresh_timer;
 };
 
 struct unlock_dialog {
@@ -174,8 +180,7 @@ check_desktop_ready(struct window *window)
 	if (!desktop->painted && is_desktop_painted(desktop)) {
 		desktop->painted = 1;
 
-		if (desktop->interface_version >= 2)
-			desktop_shell_desktop_ready(desktop->shell);
+		weston_desktop_shell_desktop_ready(desktop->shell);
 	}
 }
 
@@ -358,7 +363,7 @@ panel_clock_redraw_handler(struct widget *widget, void *data)
 
 	time(&rawtime);
 	timeinfo = localtime(&rawtime);
-	strftime(string, sizeof string, "%a %b %d, %I:%M %p", timeinfo);
+	strftime(string, sizeof string, clock->format_string, timeinfo);
 
 	widget_get_allocation(widget, &allocation);
 	if (allocation.width == 0)
@@ -387,9 +392,9 @@ clock_timer_reset(struct panel_clock *clock)
 {
 	struct itimerspec its;
 
-	its.it_interval.tv_sec = 60;
+	its.it_interval.tv_sec = clock->refresh_timer;
 	its.it_interval.tv_nsec = 0;
-	its.it_value.tv_sec = 60;
+	its.it_value.tv_sec = clock->refresh_timer;
 	its.it_value.tv_nsec = 0;
 	if (timerfd_settime(clock->clock_fd, 0, &its, NULL) < 0) {
 		fprintf(stderr, "could not set timerfd\n: %m");
@@ -409,6 +414,12 @@ panel_destroy_clock(struct panel_clock *clock)
 	free(clock);
 }
 
+enum {
+	CLOCK_FORMAT_MINUTES,
+	CLOCK_FORMAT_SECONDS,
+	CLOCK_FORMAT_NONE
+};
+
 static void
 panel_add_clock(struct panel *panel)
 {
@@ -426,6 +437,17 @@ panel_add_clock(struct panel *panel)
 	panel->clock = clock;
 	clock->clock_fd = timerfd;
 
+	switch (panel->clock_format) {
+	case CLOCK_FORMAT_MINUTES:
+		clock->format_string = "%a %b %d, %I:%M %p";
+		clock->refresh_timer = 60;
+		break;
+	case CLOCK_FORMAT_SECONDS:
+		clock->format_string = "%a %b %d, %I:%M:%S %p";
+		clock->refresh_timer = 1;
+		break;
+	}
+
 	clock->clock_task.run = clock_func;
 	display_watch_fd(window_get_display(panel->window), clock->clock_fd,
 			 EPOLLIN, &clock->clock_task);
@@ -452,8 +474,13 @@ panel_resize_handler(struct widget *widget,
 				      x, y - h / 2, w + 1, h + 1);
 		x += w + 10;
 	}
-	h=20;
-	w=170;
+
+	h = 20;
+
+	if (panel->clock_format == CLOCK_FORMAT_SECONDS)
+		w = 190;
+	else /* CLOCK_FORMAT_MINUTES */
+		w = 170;
 
 	if (panel->clock)
 		widget_set_allocation(panel->clock->widget,
@@ -462,7 +489,7 @@ panel_resize_handler(struct widget *widget,
 
 static void
 panel_configure(void *data,
-		struct desktop_shell *desktop_shell,
+		struct weston_desktop_shell *desktop_shell,
 		uint32_t edges, struct window *window,
 		int32_t width, int32_t height)
 {
@@ -494,7 +521,8 @@ panel_destroy(struct panel *panel)
 	struct panel_launcher *tmp;
 	struct panel_launcher *launcher;
 
-	panel_destroy_clock(panel->clock);
+	if (panel->clock)
+		panel_destroy_clock(panel->clock);
 
 	wl_list_for_each_safe(launcher, tmp, &panel->launcher_list, link)
 		panel_destroy_launcher(launcher);
@@ -510,6 +538,7 @@ panel_create(struct desktop *desktop)
 {
 	struct panel *panel;
 	struct weston_config_section *s;
+	char *clock_format_option = NULL;
 
 	panel = xzalloc(sizeof *panel);
 
@@ -524,11 +553,26 @@ panel_create(struct desktop *desktop)
 	widget_set_redraw_handler(panel->widget, panel_redraw_handler);
 	widget_set_resize_handler(panel->widget, panel_resize_handler);
 
-	panel_add_clock(panel);
+	s = weston_config_get_section(desktop->config, "shell", NULL, NULL);
+	weston_config_section_get_string(s, "clock-format", &clock_format_option, "");
+
+	if (strcmp(clock_format_option, "minutes") == 0)
+		panel->clock_format = CLOCK_FORMAT_MINUTES;
+	else if (strcmp(clock_format_option, "seconds") == 0)
+		panel->clock_format = CLOCK_FORMAT_SECONDS;
+	else if (strcmp(clock_format_option, "none") == 0)
+		panel->clock_format = CLOCK_FORMAT_NONE;
+	else
+		panel->clock_format = DEFAULT_CLOCK_FORMAT;
+
+	if (panel->clock_format != CLOCK_FORMAT_NONE)
+		panel_add_clock(panel);
+
+	free (clock_format_option);
 
 	s = weston_config_get_section(desktop->config, "shell", NULL, NULL);
-	weston_config_section_get_uint(s, "panel-color",
-				       &panel->color, 0xaa000000);
+	weston_config_section_get_color(s, "panel-color",
+					&panel->color, 0xaa000000);
 
 	panel_add_launchers(panel, desktop);
 
@@ -725,7 +769,7 @@ background_draw(struct widget *widget, void *data)
 
 static void
 background_configure(void *data,
-		     struct desktop_shell *desktop_shell,
+		     struct weston_desktop_shell *desktop_shell,
 		     uint32_t edges, struct window *window,
 		     int32_t width, int32_t height)
 {
@@ -860,6 +904,7 @@ unlock_dialog_create(struct desktop *desktop)
 {
 	struct display *display = desktop->display;
 	struct unlock_dialog *dialog;
+	struct wl_surface *surface;
 
 	dialog = xzalloc(sizeof *dialog);
 
@@ -884,8 +929,8 @@ unlock_dialog_create(struct desktop *desktop)
 	widget_set_touch_up_handler(dialog->button,
 				      unlock_dialog_touch_up_handler);
 
-	desktop_shell_set_lock_surface(desktop->shell,
-				       window_get_wl_surface(dialog->window));
+	surface = window_get_wl_surface(dialog->window);
+	weston_desktop_shell_set_lock_surface(desktop->shell, surface);
 
 	window_schedule_resize(dialog->window, 260, 230);
 
@@ -905,14 +950,14 @@ unlock_dialog_finish(struct task *task, uint32_t events)
 	struct desktop *desktop =
 		container_of(task, struct desktop, unlock_task);
 
-	desktop_shell_unlock(desktop->shell);
+	weston_desktop_shell_unlock(desktop->shell);
 	unlock_dialog_destroy(desktop->unlock_dialog);
 	desktop->unlock_dialog = NULL;
 }
 
 static void
 desktop_shell_configure(void *data,
-			struct desktop_shell *desktop_shell,
+			struct weston_desktop_shell *desktop_shell,
 			uint32_t edges,
 			struct wl_surface *surface,
 			int32_t width, int32_t height)
@@ -925,12 +970,12 @@ desktop_shell_configure(void *data,
 
 static void
 desktop_shell_prepare_lock_surface(void *data,
-				   struct desktop_shell *desktop_shell)
+				   struct weston_desktop_shell *desktop_shell)
 {
 	struct desktop *desktop = data;
 
 	if (!desktop->locking) {
-		desktop_shell_unlock(desktop->shell);
+		weston_desktop_shell_unlock(desktop->shell);
 		return;
 	}
 
@@ -942,52 +987,52 @@ desktop_shell_prepare_lock_surface(void *data,
 
 static void
 desktop_shell_grab_cursor(void *data,
-			  struct desktop_shell *desktop_shell,
+			  struct weston_desktop_shell *desktop_shell,
 			  uint32_t cursor)
 {
 	struct desktop *desktop = data;
 
 	switch (cursor) {
-	case DESKTOP_SHELL_CURSOR_NONE:
+	case WESTON_DESKTOP_SHELL_CURSOR_NONE:
 		desktop->grab_cursor = CURSOR_BLANK;
 		break;
-	case DESKTOP_SHELL_CURSOR_BUSY:
+	case WESTON_DESKTOP_SHELL_CURSOR_BUSY:
 		desktop->grab_cursor = CURSOR_WATCH;
 		break;
-	case DESKTOP_SHELL_CURSOR_MOVE:
+	case WESTON_DESKTOP_SHELL_CURSOR_MOVE:
 		desktop->grab_cursor = CURSOR_DRAGGING;
 		break;
-	case DESKTOP_SHELL_CURSOR_RESIZE_TOP:
+	case WESTON_DESKTOP_SHELL_CURSOR_RESIZE_TOP:
 		desktop->grab_cursor = CURSOR_TOP;
 		break;
-	case DESKTOP_SHELL_CURSOR_RESIZE_BOTTOM:
+	case WESTON_DESKTOP_SHELL_CURSOR_RESIZE_BOTTOM:
 		desktop->grab_cursor = CURSOR_BOTTOM;
 		break;
-	case DESKTOP_SHELL_CURSOR_RESIZE_LEFT:
+	case WESTON_DESKTOP_SHELL_CURSOR_RESIZE_LEFT:
 		desktop->grab_cursor = CURSOR_LEFT;
 		break;
-	case DESKTOP_SHELL_CURSOR_RESIZE_RIGHT:
+	case WESTON_DESKTOP_SHELL_CURSOR_RESIZE_RIGHT:
 		desktop->grab_cursor = CURSOR_RIGHT;
 		break;
-	case DESKTOP_SHELL_CURSOR_RESIZE_TOP_LEFT:
+	case WESTON_DESKTOP_SHELL_CURSOR_RESIZE_TOP_LEFT:
 		desktop->grab_cursor = CURSOR_TOP_LEFT;
 		break;
-	case DESKTOP_SHELL_CURSOR_RESIZE_TOP_RIGHT:
+	case WESTON_DESKTOP_SHELL_CURSOR_RESIZE_TOP_RIGHT:
 		desktop->grab_cursor = CURSOR_TOP_RIGHT;
 		break;
-	case DESKTOP_SHELL_CURSOR_RESIZE_BOTTOM_LEFT:
+	case WESTON_DESKTOP_SHELL_CURSOR_RESIZE_BOTTOM_LEFT:
 		desktop->grab_cursor = CURSOR_BOTTOM_LEFT;
 		break;
-	case DESKTOP_SHELL_CURSOR_RESIZE_BOTTOM_RIGHT:
+	case WESTON_DESKTOP_SHELL_CURSOR_RESIZE_BOTTOM_RIGHT:
 		desktop->grab_cursor = CURSOR_BOTTOM_RIGHT;
 		break;
-	case DESKTOP_SHELL_CURSOR_ARROW:
+	case WESTON_DESKTOP_SHELL_CURSOR_ARROW:
 	default:
 		desktop->grab_cursor = CURSOR_LEFT_PTR;
 	}
 }
 
-static const struct desktop_shell_listener listener = {
+static const struct weston_desktop_shell_listener listener = {
 	desktop_shell_configure,
 	desktop_shell_prepare_lock_surface,
 	desktop_shell_grab_cursor
@@ -1023,8 +1068,8 @@ background_create(struct desktop *desktop)
 	s = weston_config_get_section(desktop->config, "shell", NULL, NULL);
 	weston_config_section_get_string(s, "background-image",
 					 &background->image, NULL);
-	weston_config_section_get_uint(s, "background-color",
-				       &background->color, 0);
+	weston_config_section_get_color(s, "background-color",
+					&background->color, 0x00000000);
 
 	weston_config_section_get_string(s, "background-type",
 					 &type, "tile");
@@ -1075,7 +1120,7 @@ grab_surface_create(struct desktop *desktop)
 	window_set_user_data(desktop->grab_window, desktop);
 
 	s = window_get_wl_surface(desktop->grab_window);
-	desktop_shell_set_grab_surface(desktop->shell, s);
+	weston_desktop_shell_set_grab_surface(desktop->shell, s);
 
 	desktop->grab_widget =
 		window_add_widget(desktop->grab_window, desktop);
@@ -1189,14 +1234,14 @@ output_init(struct output *output, struct desktop *desktop)
 	if (want_panel(desktop)) {
 		output->panel = panel_create(desktop);
 		surface = window_get_wl_surface(output->panel->window);
-		desktop_shell_set_panel(desktop->shell,
-					output->output, surface);
+		weston_desktop_shell_set_panel(desktop->shell,
+					       output->output, surface);
 	}
 
 	output->background = background_create(desktop);
 	surface = window_get_wl_surface(output->background->window);
-	desktop_shell_set_background(desktop->shell,
-				     output->output, surface);
+	weston_desktop_shell_set_background(desktop->shell,
+					    output->output, surface);
 }
 
 static void
@@ -1204,7 +1249,7 @@ create_output(struct desktop *desktop, uint32_t id)
 {
 	struct output *output;
 
-	output = calloc(1, sizeof *output);
+	output = zalloc(sizeof *output);
 	if (!output)
 		return;
 
@@ -1228,12 +1273,14 @@ global_handler(struct display *display, uint32_t id,
 {
 	struct desktop *desktop = data;
 
-	if (!strcmp(interface, "desktop_shell")) {
-		desktop->interface_version = (version < 2) ? version : 2;
+	if (!strcmp(interface, "weston_desktop_shell")) {
 		desktop->shell = display_bind(desktop->display,
-					      id, &desktop_shell_interface,
-					      desktop->interface_version);
-		desktop_shell_add_listener(desktop->shell, &listener, desktop);
+					      id,
+					      &weston_desktop_shell_interface,
+					      1);
+		weston_desktop_shell_add_listener(desktop->shell,
+						  &listener,
+						  desktop);
 	} else if (!strcmp(interface, "wl_output")) {
 		create_output(desktop, id);
 	}
@@ -1334,7 +1381,7 @@ int main(int argc, char *argv[])
 	desktop_destroy_outputs(&desktop);
 	if (desktop.unlock_dialog)
 		unlock_dialog_destroy(desktop.unlock_dialog);
-	desktop_shell_destroy(desktop.shell);
+	weston_desktop_shell_destroy(desktop.shell);
 	display_destroy(desktop.display);
 
 	return 0;
